# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2017-02-12 17:48
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('plugs_media', '0008_mediacontenttype'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='media',
            name='content_type',
        ),
        migrations.AddField(
            model_name='media',
            name='media_content_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='plugs_media.MediaContentType'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2017-01-07 19:57
from __future__ import unicode_literals

import datetime
from django.db import migrations, models
from django.utils.timezone import utc


class Migration(migrations.Migration):

    dependencies = [
        ('plugs_media', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='media',
            name='created',
            field=models.DateTimeField(auto_now_add=True, default=datetime.datetime(2017, 1, 7, 19, 57, 23, 177843, tzinfo=utc)),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='media',
            name='file',
            field=models.FileField(default='', max_length=255, upload_to='%Y/%m/%d/'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='media',
            name='name',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='media',
            name='updated',
            field=models.DateTimeField(auto_now=True, default=datetime.datetime(2017, 1, 7, 19, 57, 47, 274570, tzinfo=utc)),
            preserve_default=False,
        ),
    ]



from magic import Magic

from io import BytesIO

from django.core.files.uploadedfile import InMemoryUploadedFile

from base64 import b64decode

from rest_framework import serializers

from plugs_media import models
from plugs_media import validators


class HybridFileFieldSerializer(serializers.FileField):

    def get_content_type(self, file_):
        """
        Returns the content type of the file using python magic
        """
        magic = Magic(mime=True)
        return magic.from_buffer(file_.read(1024))

    def to_base64(self, data):
        try:
            # we are discarding the data uri if present
            data = data.split(',')[1]
        except:
            pass
        file_ = BytesIO(b64decode(data))
        content_type = self.get_content_type(file_)
        extension = content_type.split('/')[1]
        return InMemoryUploadedFile(file_, None, extension, content_type, file_.__sizeof__(), None)

    def to_internal_value(self, data):
        """
        This file field serializer can accept a binary file
        or a base64 encoded string with the contents of a file
        """
        if not hasattr(data, 'file'):
            data = self.to_base64(data)
        return super().to_internal_value(data)


class MediaSerializer(serializers.ModelSerializer):
    """
    Media Serializer
    """

    file = HybridFileFieldSerializer(
        max_length=100,
        allow_empty_file=True,
        use_url=False,
        validators=[
            validators.MediaMaxSizeValidator(),
            validators.MediaContentTypeValidator(),
        ]
    )

    class Meta:
        model = models.Media
        fields = ('id', 'name', 'user', 'file', 'created', 'updated')
        read_only_fields = ('user', )



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2017-08-14 15:27
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('plugs_media', '0009_auto_20170212_1748'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='media',
            name='media_content_type',
        ),
        migrations.RemoveField(
            model_name='media',
            name='object_id',
        ),
        migrations.DeleteModel(
            name='MediaContentType',
        ),
    ]



"""
Custom fields required by media
"""

from django.db import models

from plugs_media.widgets import MediaWidget
from plugs_media.validators import MediaValidator

class MediaField(models.CharField):
    """
    This field is a special charfield
    that accepts and stores chars but
    references a resource in the media model
    which for now are images
    """

    default_validators = [MediaValidator()]

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 100
        super(MediaField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        """
        Using this to provide a custom widget
        in the admin site
        """
        kwargs['widget'] = MediaWidget
        return super(MediaField, self).formfield(**kwargs)



"""
Plugs Media Validators
"""
from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext as _
from django.core.exceptions import ValidationError

from plugs_media.models import Media
from plugs_media.settings import plugs_media_settings

# pylint: disable=R0903
class MediaValidator(object):
    """
    Validates if a string passed to a field
    accepting media resources is in fact a media
    resource
    """

    message = _('This field must be a valid media resource.')
    code = 'invalid_media'

    def __call__(self, value):
        # get the default medias
        try:
            if value in plugs_media_settings['DEFAULTS'].values():
                return
        except KeyError:
            pass
        # check if the media exists
        try:
            Media.objects.get(file=value)
        except ObjectDoesNotExist:
            raise ValidationError(self.message, code=self.code)

# pylint: disable=R0903
class MediaMaxSizeValidator(object):
    """
    Validates if file size does not exceeds the allowed limite
    """

    message = _('File size of {0} exceeds the allowed limit of {1}.')

    def __call__(self, value):
        if value.size > plugs_media_settings['MAX_FILE_SIZE']:
            raise ValidationError(self.message.format(
                value.size, plugs_media_settings['MAX_FILE_SIZE']))


# pylint: disable=R0903
class MediaContentTypeValidator(object):
    """
    Validates if file content type is allowed
    """

    message = _('File of type {0} not supported.')

    def __call__(self, value):
        if value.content_type not in plugs_media_settings['CONTENT_TYPES']:
            raise ValidationError(self.message.format(value.content_type))



"""
Media Widgets
"""

from django.conf import settings
from django.forms.widgets import ClearableFileInput

class MediaWidget(ClearableFileInput):
    """
    Media widget to deal with images
    """

    def render(self, name, value, attrs=None):
        output = super(MediaWidget, self).render(name, value, attrs)
        img_html = '<img src="{0}{1}"/><br/>'.format(settings.MEDIA_URL, value)
        return img_html + output
